"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addressinput import AddressInput, AddressInputTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


CheckoutCreateMetadataTypedDict = TypeAliasType(
    "CheckoutCreateMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutCreateMetadata = TypeAliasType(
    "CheckoutCreateMetadata", Union[str, int, float, bool]
)


CheckoutCreateCustomFieldDataTypedDict = TypeAliasType(
    "CheckoutCreateCustomFieldDataTypedDict", Union[str, int, bool, datetime]
)


CheckoutCreateCustomFieldData = TypeAliasType(
    "CheckoutCreateCustomFieldData", Union[str, int, bool, datetime]
)


CheckoutCreateCustomerMetadataTypedDict = TypeAliasType(
    "CheckoutCreateCustomerMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutCreateCustomerMetadata = TypeAliasType(
    "CheckoutCreateCustomerMetadata", Union[str, int, float, bool]
)


class CheckoutCreateTypedDict(TypedDict):
    r"""Create a new checkout session from a list of products.
    Customers will be able to switch between those products.

    Metadata set on the checkout will be copied
    to the resulting order and/or subscription.
    """

    products: List[str]
    r"""List of product IDs available to select at that checkout. The first one will be selected by default."""
    metadata: NotRequired[Dict[str, CheckoutCreateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    custom_field_data: NotRequired[
        Dict[str, Nullable[CheckoutCreateCustomFieldDataTypedDict]]
    ]
    r"""Key-value object storing custom field values."""
    discount_id: NotRequired[Nullable[str]]
    r"""ID of the discount to apply to the checkout."""
    allow_discount_codes: NotRequired[bool]
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    require_billing_address: NotRequired[bool]
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting. If you preset the billing address, this setting will be automatically set to `true`."""
    amount: NotRequired[Nullable[int]]
    customer_id: NotRequired[Nullable[str]]
    r"""ID of an existing customer in the organization. The customer data will be pre-filled in the checkout form. The resulting order will be linked to this customer."""
    is_business_customer: NotRequired[bool]
    r"""Whether the customer is a business or an individual. If `true`, the customer will be required to fill their full billing address and billing name."""
    external_customer_id: NotRequired[Nullable[str]]
    r"""ID of the customer in your system. If a matching customer exists on Polar, the resulting order will be linked to this customer. Otherwise, a new customer will be created with this external ID set."""
    customer_name: NotRequired[Nullable[str]]
    customer_email: NotRequired[Nullable[str]]
    customer_ip_address: NotRequired[Nullable[str]]
    customer_billing_name: NotRequired[Nullable[str]]
    customer_billing_address: NotRequired[Nullable[AddressInputTypedDict]]
    customer_tax_id: NotRequired[Nullable[str]]
    customer_metadata: NotRequired[Dict[str, CheckoutCreateCustomerMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information that'll be copied to the created customer.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    subscription_id: NotRequired[Nullable[str]]
    r"""ID of a subscription to upgrade. It must be on a free pricing. If checkout is successful, metadata set on this checkout will be copied to the subscription, and existing keys will be overwritten."""
    success_url: NotRequired[Nullable[str]]
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""
    embed_origin: NotRequired[Nullable[str]]
    r"""If you plan to embed the checkout session, set this to the Origin of the embedding page. It'll allow the Polar iframe to communicate with the parent page."""


class CheckoutCreate(BaseModel):
    r"""Create a new checkout session from a list of products.
    Customers will be able to switch between those products.

    Metadata set on the checkout will be copied
    to the resulting order and/or subscription.
    """

    products: List[str]
    r"""List of product IDs available to select at that checkout. The first one will be selected by default."""

    metadata: Optional[Dict[str, CheckoutCreateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    custom_field_data: Optional[Dict[str, Nullable[CheckoutCreateCustomFieldData]]] = (
        None
    )
    r"""Key-value object storing custom field values."""

    discount_id: OptionalNullable[str] = UNSET
    r"""ID of the discount to apply to the checkout."""

    allow_discount_codes: Optional[bool] = True
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    require_billing_address: Optional[bool] = False
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting. If you preset the billing address, this setting will be automatically set to `true`."""

    amount: OptionalNullable[int] = UNSET

    customer_id: OptionalNullable[str] = UNSET
    r"""ID of an existing customer in the organization. The customer data will be pre-filled in the checkout form. The resulting order will be linked to this customer."""

    is_business_customer: Optional[bool] = False
    r"""Whether the customer is a business or an individual. If `true`, the customer will be required to fill their full billing address and billing name."""

    external_customer_id: OptionalNullable[str] = UNSET
    r"""ID of the customer in your system. If a matching customer exists on Polar, the resulting order will be linked to this customer. Otherwise, a new customer will be created with this external ID set."""

    customer_name: OptionalNullable[str] = UNSET

    customer_email: OptionalNullable[str] = UNSET

    customer_ip_address: OptionalNullable[str] = UNSET

    customer_billing_name: OptionalNullable[str] = UNSET

    customer_billing_address: OptionalNullable[AddressInput] = UNSET

    customer_tax_id: OptionalNullable[str] = UNSET

    customer_metadata: Optional[Dict[str, CheckoutCreateCustomerMetadata]] = None
    r"""Key-value object allowing you to store additional information that'll be copied to the created customer.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    subscription_id: OptionalNullable[str] = UNSET
    r"""ID of a subscription to upgrade. It must be on a free pricing. If checkout is successful, metadata set on this checkout will be copied to the subscription, and existing keys will be overwritten."""

    success_url: OptionalNullable[str] = UNSET
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""

    embed_origin: OptionalNullable[str] = UNSET
    r"""If you plan to embed the checkout session, set this to the Origin of the embedding page. It'll allow the Polar iframe to communicate with the parent page."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "custom_field_data",
            "discount_id",
            "allow_discount_codes",
            "require_billing_address",
            "amount",
            "customer_id",
            "is_business_customer",
            "external_customer_id",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_name",
            "customer_billing_address",
            "customer_tax_id",
            "customer_metadata",
            "subscription_id",
            "success_url",
            "embed_origin",
        ]
        nullable_fields = [
            "discount_id",
            "amount",
            "customer_id",
            "external_customer_id",
            "customer_name",
            "customer_email",
            "customer_ip_address",
            "customer_billing_name",
            "customer_billing_address",
            "customer_tax_id",
            "subscription_id",
            "success_url",
            "embed_origin",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from polar_sdk.models import PolarError
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal, Optional
from typing_extensions import Annotated


class PaymentMethodInUseByActiveSubscriptionData(BaseModel):
    detail: str

    ERROR: Annotated[
        Annotated[
            Literal["PaymentMethodInUseByActiveSubscription"],
            AfterValidator(validate_const("PaymentMethodInUseByActiveSubscription")),
        ],
        pydantic.Field(alias="error"),
    ] = "PaymentMethodInUseByActiveSubscription"


@dataclass(frozen=True)
class PaymentMethodInUseByActiveSubscription(PolarError):
    data: PaymentMethodInUseByActiveSubscriptionData = field(hash=False)

    def __init__(
        self,
        data: PaymentMethodInUseByActiveSubscriptionData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutLinkCreateProductsMetadataTypedDict = TypeAliasType(
    "CheckoutLinkCreateProductsMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutLinkCreateProductsMetadata = TypeAliasType(
    "CheckoutLinkCreateProductsMetadata", Union[str, int, float, bool]
)


class CheckoutLinkCreateProductsTypedDict(TypedDict):
    r"""Schema to create a new checkout link."""

    products: List[str]
    r"""List of products that will be available to select at checkout."""
    metadata: NotRequired[Dict[str, CheckoutLinkCreateProductsMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    payment_processor: Literal["stripe"]
    r"""Payment processor to use. Currently only Stripe is supported."""
    label: NotRequired[Nullable[str]]
    r"""Optional label to distinguish links internally"""
    allow_discount_codes: NotRequired[bool]
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    require_billing_address: NotRequired[bool]
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting."""
    discount_id: NotRequired[Nullable[str]]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""
    success_url: NotRequired[Nullable[str]]
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""


class CheckoutLinkCreateProducts(BaseModel):
    r"""Schema to create a new checkout link."""

    products: List[str]
    r"""List of products that will be available to select at checkout."""

    metadata: Optional[Dict[str, CheckoutLinkCreateProductsMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    PAYMENT_PROCESSOR: Annotated[
        Annotated[Literal["stripe"], AfterValidator(validate_const("stripe"))],
        pydantic.Field(alias="payment_processor"),
    ] = "stripe"
    r"""Payment processor to use. Currently only Stripe is supported."""

    label: OptionalNullable[str] = UNSET
    r"""Optional label to distinguish links internally"""

    allow_discount_codes: Optional[bool] = True
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    require_billing_address: Optional[bool] = False
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting."""

    discount_id: OptionalNullable[str] = UNSET
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""

    success_url: OptionalNullable[str] = UNSET
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "label",
            "allow_discount_codes",
            "require_billing_address",
            "discount_id",
            "success_url",
        ]
        nullable_fields = ["label", "discount_id", "success_url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


CheckoutLinkUpdateMetadataTypedDict = TypeAliasType(
    "CheckoutLinkUpdateMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutLinkUpdateMetadata = TypeAliasType(
    "CheckoutLinkUpdateMetadata", Union[str, int, float, bool]
)


class CheckoutLinkUpdateTypedDict(TypedDict):
    r"""Schema to update an existing checkout link."""

    metadata: NotRequired[Dict[str, CheckoutLinkUpdateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    products: NotRequired[Nullable[List[str]]]
    r"""List of products that will be available to select at checkout."""
    label: NotRequired[Nullable[str]]
    allow_discount_codes: NotRequired[Nullable[bool]]
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    require_billing_address: NotRequired[Nullable[bool]]
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting."""
    discount_id: NotRequired[Nullable[str]]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""
    success_url: NotRequired[Nullable[str]]
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""


class CheckoutLinkUpdate(BaseModel):
    r"""Schema to update an existing checkout link."""

    metadata: Optional[Dict[str, CheckoutLinkUpdateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    products: OptionalNullable[List[str]] = UNSET
    r"""List of products that will be available to select at checkout."""

    label: OptionalNullable[str] = UNSET

    allow_discount_codes: OptionalNullable[bool] = UNSET
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    require_billing_address: OptionalNullable[bool] = UNSET
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting."""

    discount_id: OptionalNullable[str] = UNSET
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""

    success_url: OptionalNullable[str] = UNSET
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "metadata",
            "products",
            "label",
            "allow_discount_codes",
            "require_billing_address",
            "discount_id",
            "success_url",
        ]
        nullable_fields = [
            "products",
            "label",
            "allow_discount_codes",
            "require_billing_address",
            "discount_id",
            "success_url",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BillingAddressFieldMode(str, Enum):
    REQUIRED = "required"
    OPTIONAL = "optional"
    DISABLED = "disabled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


OrdersModelTypedDict = TypeAliasType("OrdersModelTypedDict", Union[int, float])


OrdersModel = TypeAliasType("OrdersModel", Union[int, float])


RevenueTypedDict = TypeAliasType("RevenueTypedDict", Union[int, float])


Revenue = TypeAliasType("Revenue", Union[int, float])


NetRevenueTypedDict = TypeAliasType("NetRevenueTypedDict", Union[int, float])


NetRevenue = TypeAliasType("NetRevenue", Union[int, float])


CumulativeRevenueTypedDict = TypeAliasType(
    "CumulativeRevenueTypedDict", Union[int, float]
)


CumulativeRevenue = TypeAliasType("CumulativeRevenue", Union[int, float])


NetCumulativeRevenueTypedDict = TypeAliasType(
    "NetCumulativeRevenueTypedDict", Union[int, float]
)


NetCumulativeRevenue = TypeAliasType("NetCumulativeRevenue", Union[int, float])


AverageOrderValueTypedDict = TypeAliasType(
    "AverageOrderValueTypedDict", Union[int, float]
)


AverageOrderValue = TypeAliasType("AverageOrderValue", Union[int, float])


NetAverageOrderValueTypedDict = TypeAliasType(
    "NetAverageOrderValueTypedDict", Union[int, float]
)


NetAverageOrderValue = TypeAliasType("NetAverageOrderValue", Union[int, float])


OneTimeProductsTypedDict = TypeAliasType("OneTimeProductsTypedDict", Union[int, float])


OneTimeProducts = TypeAliasType("OneTimeProducts", Union[int, float])


OneTimeProductsRevenueTypedDict = TypeAliasType(
    "OneTimeProductsRevenueTypedDict", Union[int, float]
)


OneTimeProductsRevenue = TypeAliasType("OneTimeProductsRevenue", Union[int, float])


OneTimeProductsNetRevenueTypedDict = TypeAliasType(
    "OneTimeProductsNetRevenueTypedDict", Union[int, float]
)


OneTimeProductsNetRevenue = TypeAliasType(
    "OneTimeProductsNetRevenue", Union[int, float]
)


NewSubscriptionsTypedDict = TypeAliasType(
    "NewSubscriptionsTypedDict", Union[int, float]
)


NewSubscriptions = TypeAliasType("NewSubscriptions", Union[int, float])


NewSubscriptionsRevenueTypedDict = TypeAliasType(
    "NewSubscriptionsRevenueTypedDict", Union[int, float]
)


NewSubscriptionsRevenue = TypeAliasType("NewSubscriptionsRevenue", Union[int, float])


NewSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "NewSubscriptionsNetRevenueTypedDict", Union[int, float]
)


NewSubscriptionsNetRevenue = TypeAliasType(
    "NewSubscriptionsNetRevenue", Union[int, float]
)


RenewedSubscriptionsTypedDict = TypeAliasType(
    "RenewedSubscriptionsTypedDict", Union[int, float]
)


RenewedSubscriptions = TypeAliasType("RenewedSubscriptions", Union[int, float])


RenewedSubscriptionsRevenueTypedDict = TypeAliasType(
    "RenewedSubscriptionsRevenueTypedDict", Union[int, float]
)


RenewedSubscriptionsRevenue = TypeAliasType(
    "RenewedSubscriptionsRevenue", Union[int, float]
)


RenewedSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "RenewedSubscriptionsNetRevenueTypedDict", Union[int, float]
)


RenewedSubscriptionsNetRevenue = TypeAliasType(
    "RenewedSubscriptionsNetRevenue", Union[int, float]
)


ActiveSubscriptionsTypedDict = TypeAliasType(
    "ActiveSubscriptionsTypedDict", Union[int, float]
)


ActiveSubscriptions = TypeAliasType("ActiveSubscriptions", Union[int, float])


MonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MonthlyRecurringRevenueTypedDict", Union[int, float]
)


MonthlyRecurringRevenue = TypeAliasType("MonthlyRecurringRevenue", Union[int, float])


CommittedMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "CommittedMonthlyRecurringRevenueTypedDict", Union[int, float]
)


CommittedMonthlyRecurringRevenue = TypeAliasType(
    "CommittedMonthlyRecurringRevenue", Union[int, float]
)


CheckoutsModelTypedDict = TypeAliasType("CheckoutsModelTypedDict", Union[int, float])


CheckoutsModel = TypeAliasType("CheckoutsModel", Union[int, float])


SucceededCheckoutsTypedDict = TypeAliasType(
    "SucceededCheckoutsTypedDict", Union[int, float]
)


SucceededCheckouts = TypeAliasType("SucceededCheckouts", Union[int, float])


CheckoutsConversionTypedDict = TypeAliasType(
    "CheckoutsConversionTypedDict", Union[int, float]
)


CheckoutsConversion = TypeAliasType("CheckoutsConversion", Union[int, float])


CanceledSubscriptionsTypedDict = TypeAliasType(
    "CanceledSubscriptionsTypedDict", Union[int, float]
)


CanceledSubscriptions = TypeAliasType("CanceledSubscriptions", Union[int, float])


CanceledSubscriptionsCustomerServiceTypedDict = TypeAliasType(
    "CanceledSubscriptionsCustomerServiceTypedDict", Union[int, float]
)


CanceledSubscriptionsCustomerService = TypeAliasType(
    "CanceledSubscriptionsCustomerService", Union[int, float]
)


CanceledSubscriptionsLowQualityTypedDict = TypeAliasType(
    "CanceledSubscriptionsLowQualityTypedDict", Union[int, float]
)


CanceledSubscriptionsLowQuality = TypeAliasType(
    "CanceledSubscriptionsLowQuality", Union[int, float]
)


CanceledSubscriptionsMissingFeaturesTypedDict = TypeAliasType(
    "CanceledSubscriptionsMissingFeaturesTypedDict", Union[int, float]
)


CanceledSubscriptionsMissingFeatures = TypeAliasType(
    "CanceledSubscriptionsMissingFeatures", Union[int, float]
)


CanceledSubscriptionsSwitchedServiceTypedDict = TypeAliasType(
    "CanceledSubscriptionsSwitchedServiceTypedDict", Union[int, float]
)


CanceledSubscriptionsSwitchedService = TypeAliasType(
    "CanceledSubscriptionsSwitchedService", Union[int, float]
)


CanceledSubscriptionsTooComplexTypedDict = TypeAliasType(
    "CanceledSubscriptionsTooComplexTypedDict", Union[int, float]
)


CanceledSubscriptionsTooComplex = TypeAliasType(
    "CanceledSubscriptionsTooComplex", Union[int, float]
)


CanceledSubscriptionsTooExpensiveTypedDict = TypeAliasType(
    "CanceledSubscriptionsTooExpensiveTypedDict", Union[int, float]
)


CanceledSubscriptionsTooExpensive = TypeAliasType(
    "CanceledSubscriptionsTooExpensive", Union[int, float]
)


CanceledSubscriptionsUnusedTypedDict = TypeAliasType(
    "CanceledSubscriptionsUnusedTypedDict", Union[int, float]
)


CanceledSubscriptionsUnused = TypeAliasType(
    "CanceledSubscriptionsUnused", Union[int, float]
)


CanceledSubscriptionsOtherTypedDict = TypeAliasType(
    "CanceledSubscriptionsOtherTypedDict", Union[int, float]
)


CanceledSubscriptionsOther = TypeAliasType(
    "CanceledSubscriptionsOther", Union[int, float]
)


class MetricPeriodTypedDict(TypedDict):
    timestamp: datetime
    r"""Timestamp of this period data."""
    orders: OrdersModelTypedDict
    revenue: RevenueTypedDict
    net_revenue: NetRevenueTypedDict
    cumulative_revenue: CumulativeRevenueTypedDict
    net_cumulative_revenue: NetCumulativeRevenueTypedDict
    average_order_value: AverageOrderValueTypedDict
    net_average_order_value: NetAverageOrderValueTypedDict
    one_time_products: OneTimeProductsTypedDict
    one_time_products_revenue: OneTimeProductsRevenueTypedDict
    one_time_products_net_revenue: OneTimeProductsNetRevenueTypedDict
    new_subscriptions: NewSubscriptionsTypedDict
    new_subscriptions_revenue: NewSubscriptionsRevenueTypedDict
    new_subscriptions_net_revenue: NewSubscriptionsNetRevenueTypedDict
    renewed_subscriptions: RenewedSubscriptionsTypedDict
    renewed_subscriptions_revenue: RenewedSubscriptionsRevenueTypedDict
    renewed_subscriptions_net_revenue: RenewedSubscriptionsNetRevenueTypedDict
    active_subscriptions: ActiveSubscriptionsTypedDict
    monthly_recurring_revenue: MonthlyRecurringRevenueTypedDict
    committed_monthly_recurring_revenue: CommittedMonthlyRecurringRevenueTypedDict
    checkouts: CheckoutsModelTypedDict
    succeeded_checkouts: SucceededCheckoutsTypedDict
    checkouts_conversion: CheckoutsConversionTypedDict
    canceled_subscriptions: CanceledSubscriptionsTypedDict
    canceled_subscriptions_customer_service: (
        CanceledSubscriptionsCustomerServiceTypedDict
    )
    canceled_subscriptions_low_quality: CanceledSubscriptionsLowQualityTypedDict
    canceled_subscriptions_missing_features: (
        CanceledSubscriptionsMissingFeaturesTypedDict
    )
    canceled_subscriptions_switched_service: (
        CanceledSubscriptionsSwitchedServiceTypedDict
    )
    canceled_subscriptions_too_complex: CanceledSubscriptionsTooComplexTypedDict
    canceled_subscriptions_too_expensive: CanceledSubscriptionsTooExpensiveTypedDict
    canceled_subscriptions_unused: CanceledSubscriptionsUnusedTypedDict
    canceled_subscriptions_other: CanceledSubscriptionsOtherTypedDict


class MetricPeriod(BaseModel):
    timestamp: datetime
    r"""Timestamp of this period data."""

    orders: OrdersModel

    revenue: Revenue

    net_revenue: NetRevenue

    cumulative_revenue: CumulativeRevenue

    net_cumulative_revenue: NetCumulativeRevenue

    average_order_value: AverageOrderValue

    net_average_order_value: NetAverageOrderValue

    one_time_products: OneTimeProducts

    one_time_products_revenue: OneTimeProductsRevenue

    one_time_products_net_revenue: OneTimeProductsNetRevenue

    new_subscriptions: NewSubscriptions

    new_subscriptions_revenue: NewSubscriptionsRevenue

    new_subscriptions_net_revenue: NewSubscriptionsNetRevenue

    renewed_subscriptions: RenewedSubscriptions

    renewed_subscriptions_revenue: RenewedSubscriptionsRevenue

    renewed_subscriptions_net_revenue: RenewedSubscriptionsNetRevenue

    active_subscriptions: ActiveSubscriptions

    monthly_recurring_revenue: MonthlyRecurringRevenue

    committed_monthly_recurring_revenue: CommittedMonthlyRecurringRevenue

    checkouts: CheckoutsModel

    succeeded_checkouts: SucceededCheckouts

    checkouts_conversion: CheckoutsConversion

    canceled_subscriptions: CanceledSubscriptions

    canceled_subscriptions_customer_service: CanceledSubscriptionsCustomerService

    canceled_subscriptions_low_quality: CanceledSubscriptionsLowQuality

    canceled_subscriptions_missing_features: CanceledSubscriptionsMissingFeatures

    canceled_subscriptions_switched_service: CanceledSubscriptionsSwitchedService

    canceled_subscriptions_too_complex: CanceledSubscriptionsTooComplex

    canceled_subscriptions_too_expensive: CanceledSubscriptionsTooExpensive

    canceled_subscriptions_unused: CanceledSubscriptionsUnused

    canceled_subscriptions_other: CanceledSubscriptionsOther



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addressinput import AddressInput, AddressInputTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


CheckoutUpdateCustomFieldDataTypedDict = TypeAliasType(
    "CheckoutUpdateCustomFieldDataTypedDict", Union[str, int, bool, datetime]
)


CheckoutUpdateCustomFieldData = TypeAliasType(
    "CheckoutUpdateCustomFieldData", Union[str, int, bool, datetime]
)


CheckoutUpdateMetadataTypedDict = TypeAliasType(
    "CheckoutUpdateMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutUpdateMetadata = TypeAliasType(
    "CheckoutUpdateMetadata", Union[str, int, float, bool]
)


CheckoutUpdateCustomerMetadataTypedDict = TypeAliasType(
    "CheckoutUpdateCustomerMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutUpdateCustomerMetadata = TypeAliasType(
    "CheckoutUpdateCustomerMetadata", Union[str, int, float, bool]
)


class CheckoutUpdateTypedDict(TypedDict):
    r"""Update an existing checkout session using an access token."""

    custom_field_data: NotRequired[
        Dict[str, Nullable[CheckoutUpdateCustomFieldDataTypedDict]]
    ]
    r"""Key-value object storing custom field values."""
    product_id: NotRequired[Nullable[str]]
    r"""ID of the product to checkout. Must be present in the checkout's product list."""
    product_price_id: NotRequired[Nullable[str]]
    r"""ID of the product price to checkout. Must correspond to a price present in the checkout's product list."""
    amount: NotRequired[Nullable[int]]
    is_business_customer: NotRequired[Nullable[bool]]
    customer_name: NotRequired[Nullable[str]]
    customer_email: NotRequired[Nullable[str]]
    customer_billing_name: NotRequired[Nullable[str]]
    customer_billing_address: NotRequired[Nullable[AddressInputTypedDict]]
    customer_tax_id: NotRequired[Nullable[str]]
    metadata: NotRequired[Dict[str, CheckoutUpdateMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    discount_id: NotRequired[Nullable[str]]
    r"""ID of the discount to apply to the checkout."""
    allow_discount_codes: NotRequired[Nullable[bool]]
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    require_billing_address: NotRequired[Nullable[bool]]
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting. If you preset the billing address, this setting will be automatically set to `true`."""
    customer_ip_address: NotRequired[Nullable[str]]
    customer_metadata: NotRequired[
        Nullable[Dict[str, CheckoutUpdateCustomerMetadataTypedDict]]
    ]
    r"""Key-value object allowing you to store additional information that'll be copied to the created customer.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    success_url: NotRequired[Nullable[str]]
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""
    embed_origin: NotRequired[Nullable[str]]
    r"""If you plan to embed the checkout session, set this to the Origin of the embedding page. It'll allow the Polar iframe to communicate with the parent page."""


class CheckoutUpdate(BaseModel):
    r"""Update an existing checkout session using an access token."""

    custom_field_data: Optional[Dict[str, Nullable[CheckoutUpdateCustomFieldData]]] = (
        None
    )
    r"""Key-value object storing custom field values."""

    product_id: OptionalNullable[str] = UNSET
    r"""ID of the product to checkout. Must be present in the checkout's product list."""

    product_price_id: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""ID of the product price to checkout. Must correspond to a price present in the checkout's product list."""

    amount: OptionalNullable[int] = UNSET

    is_business_customer: OptionalNullable[bool] = UNSET

    customer_name: OptionalNullable[str] = UNSET

    customer_email: OptionalNullable[str] = UNSET

    customer_billing_name: OptionalNullable[str] = UNSET

    customer_billing_address: OptionalNullable[AddressInput] = UNSET

    customer_tax_id: OptionalNullable[str] = UNSET

    metadata: Optional[Dict[str, CheckoutUpdateMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    discount_id: OptionalNullable[str] = UNSET
    r"""ID of the discount to apply to the checkout."""

    allow_discount_codes: OptionalNullable[bool] = UNSET
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    require_billing_address: OptionalNullable[bool] = UNSET
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting. If you preset the billing address, this setting will be automatically set to `true`."""

    customer_ip_address: OptionalNullable[str] = UNSET

    customer_metadata: OptionalNullable[Dict[str, CheckoutUpdateCustomerMetadata]] = (
        UNSET
    )
    r"""Key-value object allowing you to store additional information that'll be copied to the created customer.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    success_url: OptionalNullable[str] = UNSET
    r"""URL where the customer will be redirected after a successful payment.You can add the `checkout_id={CHECKOUT_ID}` query parameter to retrieve the checkout session id."""

    embed_origin: OptionalNullable[str] = UNSET
    r"""If you plan to embed the checkout session, set this to the Origin of the embedding page. It'll allow the Polar iframe to communicate with the parent page."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "custom_field_data",
            "product_id",
            "product_price_id",
            "amount",
            "is_business_customer",
            "customer_name",
            "customer_email",
            "customer_billing_name",
            "customer_billing_address",
            "customer_tax_id",
            "metadata",
            "discount_id",
            "allow_discount_codes",
            "require_billing_address",
            "customer_ip_address",
            "customer_metadata",
            "success_url",
            "embed_origin",
        ]
        nullable_fields = [
            "product_id",
            "product_price_id",
            "amount",
            "is_business_customer",
            "customer_name",
            "customer_email",
            "customer_billing_name",
            "customer_billing_address",
            "customer_tax_id",
            "discount_id",
            "allow_discount_codes",
            "require_billing_address",
            "customer_ip_address",
            "customer_metadata",
            "success_url",
            "embed_origin",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantMeterCreditPropertiesTypedDict(TypedDict):
    last_credited_meter_id: NotRequired[str]
    last_credited_units: NotRequired[int]
    last_credited_at: NotRequired[str]


class BenefitGrantMeterCreditProperties(BaseModel):
    last_credited_meter_id: Optional[str] = None

    last_credited_units: Optional[int] = None

    last_credited_at: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingaddressfieldmode import BillingAddressFieldMode
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CheckoutBillingAddressFieldsTypedDict(TypedDict):
    country: BillingAddressFieldMode
    state: BillingAddressFieldMode
    city: BillingAddressFieldMode
    postal_code: BillingAddressFieldMode
    line1: BillingAddressFieldMode
    line2: BillingAddressFieldMode


class CheckoutBillingAddressFields(BaseModel):
    country: BillingAddressFieldMode

    state: BillingAddressFieldMode

    city: BillingAddressFieldMode

    postal_code: BillingAddressFieldMode

    line1: BillingAddressFieldMode

    line2: BillingAddressFieldMode



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .checkoutlinkproduct import CheckoutLinkProduct, CheckoutLinkProductTypedDict
from .discountfixedonceforeverdurationbase import (
    DiscountFixedOnceForeverDurationBase,
    DiscountFixedOnceForeverDurationBaseTypedDict,
)
from .discountfixedrepeatdurationbase import (
    DiscountFixedRepeatDurationBase,
    DiscountFixedRepeatDurationBaseTypedDict,
)
from .discountpercentageonceforeverdurationbase import (
    DiscountPercentageOnceForeverDurationBase,
    DiscountPercentageOnceForeverDurationBaseTypedDict,
)
from .discountpercentagerepeatdurationbase import (
    DiscountPercentageRepeatDurationBase,
    DiscountPercentageRepeatDurationBaseTypedDict,
)
from .paymentprocessor import PaymentProcessor
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Union
from typing_extensions import TypeAliasType, TypedDict


CheckoutLinkMetadataTypedDict = TypeAliasType(
    "CheckoutLinkMetadataTypedDict", Union[str, int, float, bool]
)


CheckoutLinkMetadata = TypeAliasType(
    "CheckoutLinkMetadata", Union[str, int, float, bool]
)


CheckoutLinkDiscountTypedDict = TypeAliasType(
    "CheckoutLinkDiscountTypedDict",
    Union[
        DiscountPercentageOnceForeverDurationBaseTypedDict,
        DiscountFixedOnceForeverDurationBaseTypedDict,
        DiscountPercentageRepeatDurationBaseTypedDict,
        DiscountFixedRepeatDurationBaseTypedDict,
    ],
)


CheckoutLinkDiscount = TypeAliasType(
    "CheckoutLinkDiscount",
    Union[
        DiscountPercentageOnceForeverDurationBase,
        DiscountFixedOnceForeverDurationBase,
        DiscountPercentageRepeatDurationBase,
        DiscountFixedRepeatDurationBase,
    ],
)


class CheckoutLinkTypedDict(TypedDict):
    r"""Checkout link data."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    metadata: Dict[str, CheckoutLinkMetadataTypedDict]
    payment_processor: PaymentProcessor
    client_secret: str
    r"""Client secret used to access the checkout link."""
    success_url: Nullable[str]
    r"""URL where the customer will be redirected after a successful payment."""
    label: Nullable[str]
    r"""Optional label to distinguish links internally"""
    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""
    require_billing_address: bool
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting."""
    discount_id: Nullable[str]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""
    organization_id: str
    r"""The organization ID."""
    products: List[CheckoutLinkProductTypedDict]
    discount: Nullable[CheckoutLinkDiscountTypedDict]
    url: str


class CheckoutLink(BaseModel):
    r"""Checkout link data."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    metadata: Dict[str, CheckoutLinkMetadata]

    payment_processor: PaymentProcessor

    client_secret: str
    r"""Client secret used to access the checkout link."""

    success_url: Nullable[str]
    r"""URL where the customer will be redirected after a successful payment."""

    label: Nullable[str]
    r"""Optional label to distinguish links internally"""

    allow_discount_codes: bool
    r"""Whether to allow the customer to apply discount codes. If you apply a discount through `discount_id`, it'll still be applied, but the customer won't be able to change it."""

    require_billing_address: bool
    r"""Whether to require the customer to fill their full billing address, instead of just the country. Customers in the US will always be required to fill their full address, regardless of this setting."""

    discount_id: Nullable[str]
    r"""ID of the discount to apply to the checkout. If the discount is not applicable anymore when opening the checkout link, it'll be ignored."""

    organization_id: str
    r"""The organization ID."""

    products: List[CheckoutLinkProduct]

    discount: Nullable[CheckoutLinkDiscount]

    url: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "success_url",
            "label",
            "discount_id",
            "discount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pagination import Pagination, PaginationTypedDict
from .product import Product, ProductTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListResourceProductTypedDict(TypedDict):
    items: List[ProductTypedDict]
    pagination: PaginationTypedDict


class ListResourceProduct(BaseModel):
    items: List[Product]

    pagination: Pagination



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SubscriptionProductUpdatedMetadataTypedDict(TypedDict):
    subscription_id: str
    old_product_id: str
    new_product_id: str


class SubscriptionProductUpdatedMetadata(BaseModel):
    subscription_id: str

    old_product_id: str

    new_product_id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customerordersortproperty import CustomerOrderSortProperty
from .listresource_customerorder_ import (
    ListResourceCustomerOrder,
    ListResourceCustomerOrderTypedDict,
)
from .productbillingtype import ProductBillingType
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata, SecurityMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class CustomerPortalOrdersListSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalOrdersListSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


CustomerPortalOrdersListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "CustomerPortalOrdersListQueryParamOrganizationIDFilterTypedDict",
    Union[str, List[str]],
)
r"""Filter by organization ID."""


CustomerPortalOrdersListQueryParamOrganizationIDFilter = TypeAliasType(
    "CustomerPortalOrdersListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


CustomerPortalOrdersListQueryParamProductIDFilterTypedDict = TypeAliasType(
    "CustomerPortalOrdersListQueryParamProductIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by product ID."""


CustomerPortalOrdersListQueryParamProductIDFilter = TypeAliasType(
    "CustomerPortalOrdersListQueryParamProductIDFilter", Union[str, List[str]]
)
r"""Filter by product ID."""


CustomerPortalOrdersListQueryParamProductBillingTypeFilterTypedDict = TypeAliasType(
    "CustomerPortalOrdersListQueryParamProductBillingTypeFilterTypedDict",
    Union[ProductBillingType, List[ProductBillingType]],
)
r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


CustomerPortalOrdersListQueryParamProductBillingTypeFilter = TypeAliasType(
    "CustomerPortalOrdersListQueryParamProductBillingTypeFilter",
    Union[ProductBillingType, List[ProductBillingType]],
)
r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""


CustomerPortalOrdersListQueryParamSubscriptionIDFilterTypedDict = TypeAliasType(
    "CustomerPortalOrdersListQueryParamSubscriptionIDFilterTypedDict",
    Union[str, List[str]],
)
r"""Filter by subscription ID."""


CustomerPortalOrdersListQueryParamSubscriptionIDFilter = TypeAliasType(
    "CustomerPortalOrdersListQueryParamSubscriptionIDFilter", Union[str, List[str]]
)
r"""Filter by subscription ID."""


class CustomerPortalOrdersListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[CustomerPortalOrdersListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    product_id: NotRequired[
        Nullable[CustomerPortalOrdersListQueryParamProductIDFilterTypedDict]
    ]
    r"""Filter by product ID."""
    product_billing_type: NotRequired[
        Nullable[CustomerPortalOrdersListQueryParamProductBillingTypeFilterTypedDict]
    ]
    r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""
    subscription_id: NotRequired[
        Nullable[CustomerPortalOrdersListQueryParamSubscriptionIDFilterTypedDict]
    ]
    r"""Filter by subscription ID."""
    query: NotRequired[Nullable[str]]
    r"""Search by product or organization name."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[CustomerOrderSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class CustomerPortalOrdersListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[CustomerPortalOrdersListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    product_id: Annotated[
        OptionalNullable[CustomerPortalOrdersListQueryParamProductIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product ID."""

    product_billing_type: Annotated[
        OptionalNullable[CustomerPortalOrdersListQueryParamProductBillingTypeFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by product billing type. `recurring` will filter data corresponding to subscriptions creations or renewals. `one_time` will filter data corresponding to one-time purchases."""

    subscription_id: Annotated[
        OptionalNullable[CustomerPortalOrdersListQueryParamSubscriptionIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by subscription ID."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Search by product or organization name."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[CustomerOrderSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "product_id",
            "product_billing_type",
            "subscription_id",
            "query",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "organization_id",
            "product_id",
            "product_billing_type",
            "subscription_id",
            "query",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CustomerPortalOrdersListResponseTypedDict(TypedDict):
    result: ListResourceCustomerOrderTypedDict


class CustomerPortalOrdersListResponse(BaseModel):
    next: Callable[[], Optional[CustomerPortalOrdersListResponse]]

    result: ListResourceCustomerOrder



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .addressinput import AddressInput, AddressInputTypedDict
from .taxidformat import TaxIDFormat
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


CustomerUpdateExternalIDMetadataTypedDict = TypeAliasType(
    "CustomerUpdateExternalIDMetadataTypedDict", Union[str, int, float, bool]
)


CustomerUpdateExternalIDMetadata = TypeAliasType(
    "CustomerUpdateExternalIDMetadata", Union[str, int, float, bool]
)


CustomerUpdateExternalIDTaxIDTypedDict = TypeAliasType(
    "CustomerUpdateExternalIDTaxIDTypedDict", Union[str, TaxIDFormat]
)


CustomerUpdateExternalIDTaxID = TypeAliasType(
    "CustomerUpdateExternalIDTaxID", Union[str, TaxIDFormat]
)


class CustomerUpdateExternalIDTypedDict(TypedDict):
    metadata: NotRequired[Dict[str, CustomerUpdateExternalIDMetadataTypedDict]]
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """
    email: NotRequired[Nullable[str]]
    r"""The email address of the customer. This must be unique within the organization."""
    name: NotRequired[Nullable[str]]
    r"""The name of the customer."""
    billing_address: NotRequired[Nullable[AddressInputTypedDict]]
    tax_id: NotRequired[
        Nullable[List[Nullable[CustomerUpdateExternalIDTaxIDTypedDict]]]
    ]


class CustomerUpdateExternalID(BaseModel):
    metadata: Optional[Dict[str, CustomerUpdateExternalIDMetadata]] = None
    r"""Key-value object allowing you to store additional information.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    email: OptionalNullable[str] = UNSET
    r"""The email address of the customer. This must be unique within the organization."""

    name: OptionalNullable[str] = UNSET
    r"""The name of the customer."""

    billing_address: OptionalNullable[AddressInput] = UNSET

    tax_id: OptionalNullable[List[Nullable[CustomerUpdateExternalIDTaxID]]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["metadata", "email", "name", "billing_address", "tax_id"]
        nullable_fields = ["email", "name", "billing_address", "tax_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationfeaturesettings import (
    OrganizationFeatureSettings,
    OrganizationFeatureSettingsTypedDict,
)
from .organizationnotificationsettings import (
    OrganizationNotificationSettings,
    OrganizationNotificationSettingsTypedDict,
)
from .organizationsociallink import (
    OrganizationSocialLink,
    OrganizationSocialLinkTypedDict,
)
from .organizationsubscriptionsettings import (
    OrganizationSubscriptionSettings,
    OrganizationSubscriptionSettingsTypedDict,
)
from .status import Status
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import TypedDict


class OrganizationTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The organization ID."""
    name: str
    r"""Organization name shown in checkout, customer portal, emails etc."""
    slug: str
    r"""Unique organization slug in checkout, customer portal and credit card statements."""
    avatar_url: Nullable[str]
    r"""Avatar URL shown in checkout, customer portal, emails etc."""
    email: Nullable[str]
    r"""Public support email."""
    website: Nullable[str]
    r"""Official website of the organization."""
    socials: List[OrganizationSocialLinkTypedDict]
    r"""Links to social profiles."""
    status: Status
    details_submitted_at: Nullable[datetime]
    r"""When the business details were submitted."""
    feature_settings: Nullable[OrganizationFeatureSettingsTypedDict]
    r"""Organization feature settings"""
    subscription_settings: OrganizationSubscriptionSettingsTypedDict
    notification_settings: OrganizationNotificationSettingsTypedDict


class Organization(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The organization ID."""

    name: str
    r"""Organization name shown in checkout, customer portal, emails etc."""

    slug: str
    r"""Unique organization slug in checkout, customer portal and credit card statements."""

    avatar_url: Nullable[str]
    r"""Avatar URL shown in checkout, customer portal, emails etc."""

    email: Nullable[str]
    r"""Public support email."""

    website: Nullable[str]
    r"""Official website of the organization."""

    socials: List[OrganizationSocialLink]
    r"""Links to social profiles."""

    status: Status

    details_submitted_at: Nullable[datetime]
    r"""When the business details were submitted."""

    feature_settings: Nullable[OrganizationFeatureSettings]
    r"""Organization feature settings"""

    subscription_settings: OrganizationSubscriptionSettings

    notification_settings: OrganizationNotificationSettings

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "avatar_url",
            "email",
            "website",
            "details_submitted_at",
            "feature_settings",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customer import Customer, CustomerTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


UserEventMetadataTypedDict = TypeAliasType(
    "UserEventMetadataTypedDict", Union[str, int, float, bool]
)


UserEventMetadata = TypeAliasType("UserEventMetadata", Union[str, int, float, bool])


class UserEventTypedDict(TypedDict):
    r"""An event you created through the ingestion API."""

    metadata: Dict[str, UserEventMetadataTypedDict]
    id: str
    r"""The ID of the object."""
    timestamp: datetime
    r"""The timestamp of the event."""
    organization_id: str
    r"""The ID of the organization owning the event."""
    customer_id: Nullable[str]
    r"""ID of the customer in your Polar organization associated with the event."""
    customer: Nullable[CustomerTypedDict]
    r"""The customer associated with the event."""
    external_customer_id: Nullable[str]
    r"""ID of the customer in your system associated with the event."""
    name: str
    r"""The name of the event."""
    source: Literal["user"]
    r"""The source of the event. `system` events are created by Polar. `user` events are the one you create through our ingestion API."""


class UserEvent(BaseModel):
    r"""An event you created through the ingestion API."""

    metadata: Dict[str, UserEventMetadata]

    id: str
    r"""The ID of the object."""

    timestamp: datetime
    r"""The timestamp of the event."""

    organization_id: str
    r"""The ID of the organization owning the event."""

    customer_id: Nullable[str]
    r"""ID of the customer in your Polar organization associated with the event."""

    customer: Nullable[Customer]
    r"""The customer associated with the event."""

    external_customer_id: Nullable[str]
    r"""ID of the customer in your system associated with the event."""

    name: str
    r"""The name of the event."""

    SOURCE: Annotated[
        Annotated[Literal["user"], AfterValidator(validate_const("user"))],
        pydantic.Field(alias="source"),
    ] = "user"
    r"""The source of the event. `system` events are created by Polar. `user` events are the one you create through our ingestion API."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["customer_id", "customer", "external_customer_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class BenefitGrantErrorTypedDict(TypedDict):
    message: str
    type: str
    timestamp: str


class BenefitGrantError(BaseModel):
    message: str

    type: str

    timestamp: str
